#!/usr/bin/env python

from datetime import datetime, timedelta
from typing import Iterable, Optional

import boto3
from botocore.config import Config


def get_s3_bucket_region(bucket_name: str):
    s3_client = boto3.client("s3", config=Config(retries={"mode": "standard"}))
    region_name = "us-east-1"
    location_res = s3_client.get_bucket_location(Bucket=bucket_name)
    if "LocationConstraint" in location_res and location_res["LocationConstraint"]:
        region_name = location_res["LocationConstraint"]
    return region_name


def get_s3_bucket_size_bytes(bucket_name: str, region_name: str) -> Optional[int]:
    end_datetime = datetime.now()
    start_datetime = end_datetime - timedelta(days=2)
    cw_client = boto3.client("cloudwatch", region_name=region_name, config=Config(retries={"mode": "standard"}))
    response = cw_client.get_metric_statistics(
        Namespace="AWS/S3",
        MetricName="BucketSizeBytes",
        Dimensions=[
            {"Name": "BucketName", "Value": bucket_name},
            {"Name": "StorageType", "Value": "StandardStorage"},
        ],
        Statistics=["Maximum"],
        StartTime=start_datetime,
        EndTime=end_datetime,
        Period=86400,  # daily
    )
    datapoints = sorted(response["Datapoints"], key=lambda x: x["Timestamp"])
    if len(datapoints) == 0:
        return None
    return int(datapoints[-1]["Maximum"])  # latest value


def get_s3_bucket_names() -> Iterable[str]:
    s3_client = boto3.client("s3", config=Config(retries={"mode": "standard"}))
    params = {}
    while True:
        res = s3_client.list_buckets(**params)
        for bucket in res["Buckets"]:
            yield bucket["Name"]
        if "ContinuationToken" not in res:
            break
        params["ContinuationToken"] = res["ContinuationToken"]


def run():
    for bucket_name in get_s3_bucket_names():
        size = get_s3_bucket_size_bytes(bucket_name, get_s3_bucket_region(bucket_name))
        if size is None:
            continue
        print(f"{size},{bucket_name}")


def main():
    run()


if __name__ == "__main__":
    main()
